@@ -18,6 +18,7 @@
 //! ```
 use std::cmp::Ordering;
 use std::mem;
+use std::{iter, vec};
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub struct NonEmpty<T>(T, Vec<T>);
@@ -195,7 +196,7 @@ impl<T> NonEmpty<T> {
     /// assert_eq!(l_iter.next(), None);
     /// ```
     pub fn iter<'a>(&'a self) -> impl Iterator<Item = &T> + 'a {
-        std::iter::once(&self.0).chain(self.1.iter())
+        iter::once(&self.0).chain(self.1.iter())
     }
 
     /// ```
@@ -217,7 +218,7 @@ impl<T> NonEmpty<T> {
     /// assert_eq!(l_iter.next(), None);
     /// ```
     pub fn iter_mut<'a>(&'a mut self) -> impl Iterator<Item = &mut T> + 'a {
-        std::iter::once(&mut self.0).chain(self.1.iter_mut())
+        iter::once(&mut self.0).chain(self.1.iter_mut())
     }
 
     /// Often we have a `Vec` (or slice `&[T]`) but want to ensure that it is `NonEmpty` before
@@ -245,6 +246,34 @@ impl<T> NonEmpty<T> {
             .map(|(h, t)| NonEmpty(h.clone(), t.into()))
     }
 
+    /// Often we have a `Vec` (or slice `&[T]`) but want to ensure that it is `NonEmpty` before
+    /// proceeding with a computation. Using `from_vec` will give us a proof
+    /// that we have a `NonEmpty` in the `Some` branch, otherwise it allows
+    /// the caller to handle the `None` case.
+    ///
+    /// This version will consume the `Vec` you pass in. If you would rather pass the data as a
+    /// slice then use `NonEmpty::from_slice`.
+    ///
+    /// # Example Use
+    ///
+    /// ```
+    /// use nonempty::NonEmpty;
+    ///
+    /// let non_empty_vec = NonEmpty::from_vec(vec![1, 2, 3, 4, 5]);
+    /// assert_eq!(non_empty_vec, Some(NonEmpty::from((1, vec![2, 3, 4, 5]))));
+    ///
+    /// let empty_vec: Option<NonEmpty<&u32>> = NonEmpty::from_vec(vec![]);
+    /// assert!(empty_vec.is_none());
+    /// ```
+    pub fn from_vec(mut vec: Vec<T>) -> Option<NonEmpty<T>> {
+        if vec.is_empty() {
+            None
+        } else {
+            let head = vec.remove(0);
+            Some(NonEmpty(head, vec))
+        }
+    }
+
     /// Deconstruct a `NonEmpty` into its head and tail.
     /// This operation never fails since we are guranteed
     /// to have a head element.
@@ -333,11 +362,11 @@ impl<T> NonEmpty<T> {
     ///
     /// assert_eq!(squares, expected);
     /// ```
-    pub fn map<U, F>(&self, f: F) -> NonEmpty<U>
+    pub fn map<U, F>(self, mut f: F) -> NonEmpty<U>
     where
-        F: Fn(&T) -> U,
+        F: FnMut(T) -> U,
     {
-        NonEmpty(f(&self.0), self.1.iter().map(f).collect())
+        NonEmpty(f(self.0), self.1.into_iter().map(f).collect())
     }
 
     /// Binary searches this sorted non-empty vector for a given element.
@@ -388,7 +417,7 @@ impl<T> NonEmpty<T> {
     /// If the value is found then Result::Ok is returned, containing the index of the matching element.
     /// If there are multiple matches, then any one of the matches could be returned.
     /// If the value is not found then Result::Err is returned, containing the index where a matching element could be
-    ///	inserted while maintaining sorted order.
+    /// inserted while maintaining sorted order.
     ///
     /// # Examples
     ///
@@ -606,10 +635,17 @@ impl<T> NonEmpty<T> {
     }
 }
 
-impl<T> Into<Vec<T>> for NonEmpty<T> {
+impl<T> From<NonEmpty<T>> for Vec<T> {
     /// Turns a non-empty list into a Vec.
-    fn into(self) -> Vec<T> {
-        std::iter::once(self.0).chain(self.1).collect()
+    fn from(nonempty: NonEmpty<T>) -> Vec<T> {
+        iter::once(nonempty.0).chain(nonempty.1).collect()
+    }
+}
+
+impl<T> From<NonEmpty<T>> for (T, Vec<T>) {
+    /// Turns a non-empty list into a Vec.
+    fn from(nonempty: NonEmpty<T>) -> (T, Vec<T>) {
+        (nonempty.0, nonempty.1)
     }
 }
 
@@ -621,6 +657,15 @@ impl<T> From<(T, Vec<T>)> for NonEmpty<T> {
     }
 }
 
+impl<T> IntoIterator for NonEmpty<T> {
+    type Item = T;
+    type IntoIter = iter::Chain<iter::Once<T>, vec::IntoIter<Self::Item>>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        iter::once(self.0).chain(self.1)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::NonEmpty;
@@ -631,4 +676,12 @@ mod tests {
         let expected = NonEmpty(1, vec![2, 3, 4, 5]);
         assert_eq!(result, expected);
     }
+
+    #[test]
+    fn test_into_iter() {
+        let nonempty = NonEmpty::from((0, vec![1, 2, 3]));
+        for (i, n) in nonempty.into_iter().enumerate() {
+            assert_eq!(i as i32, n);
+        }
+    }
 }
